"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const serialport_1 = __importDefault(require("serialport"));
const parser_1 = require("./parser");
const constants_1 = require("./constants");
const util_1 = __importDefault(require("util"));
const debug = require('debug')('prgx:connection');
class Connection {
    constructor(portPath) {
        this.portPath = portPath;
        this.tx = new serialport_1.default(portPath, { autoOpen: false, baudRate: constants_1.BAUDRATE });
        this.rx = this.tx.pipe(new parser_1.PacketParser());
    }
    open() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.openPort();
            this.tx.write(constants_1.START_BUFFER) || this.tx.drain();
            const response = yield this.receive();
            return Buffer.from(response).equals(constants_1.OPEN_BUFFER);
        });
    }
    close() {
        this.tx.close(() => debug(`Connection closed at ${this.portPath}`));
    }
    send(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const packet = this.createPacket(data);
            if (!this.tx.write(packet)) {
                this.tx.drain();
            }
            return yield this.receive();
        });
    }
    openPort() {
        return new Promise((resolve, reject) => {
            this.tx.open((err) => {
                if (err) {
                    debug(`Unable to open port: ${err.message}`);
                    reject(err);
                }
                debug(`Successfully open port ${this.portPath}`);
                resolve();
            });
        });
    }
    receive() {
        return new Promise((resolve) => {
            this.rx.once('data', (data) => {
                const packet = this.removeEscape(data);
                debug(`Received packet: ${util_1.default.inspect(packet)}`);
                resolve(packet);
            });
            //TODO: handle timeout
        });
    }
    createPacket(data) {
        const payload = data.reduce((acc, byte) => {
            if ([constants_1.STX, constants_1.ETX, constants_1.ESC].includes(byte)) {
                return acc.concat([constants_1.ESC, byte | 0x80]);
            }
            return acc.concat(byte);
        }, []);
        return [constants_1.STX, ...payload, constants_1.ETX];
    }
    removeEscape(packet) {
        let offset = 0x00;
        return packet.reduce((acc, byte) => {
            if (byte === constants_1.ESC) {
                offset = 0x80;
                return acc;
            }
            acc.push(byte - offset);
            offset = 0x00;
            return acc;
        }, []);
    }
}
exports.Connection = Connection;
