"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = __importDefault(require("util"));
const connection_1 = require("./connection");
const constants_1 = require("./constants");
const crc = __importStar(require("./crc"));
const debug = require('debug')('prgx:programmer');
class Programmer {
    constructor(portPath) {
        this.connection = new connection_1.Connection(portPath);
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.connection.open();
            const { model, version: hwVersion } = yield this.getBoardInfo();
            const fwVersion = yield this.getFirmwareVersion();
            return { model, hwVersion, fwVersion };
        });
    }
    flash(address, data) {
        var e_1, _a;
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setFlashAddress(address);
            const chunks = this.split(Array.from(data));
            try {
                for (var chunks_1 = __asyncValues(chunks), chunks_1_1; chunks_1_1 = yield chunks_1.next(), !chunks_1_1.done;) {
                    const chunk = chunks_1_1.value;
                    yield this.writeFlash(chunk);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (chunks_1_1 && !chunks_1_1.done && (_a = chunks_1.return)) yield _a.call(chunks_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            // Send last command to force programming of last flash page (see manual for details)
            yield this.writeFlash([]);
        });
    }
    quit() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.startApplication();
            this.connection.close();
        });
    }
    startApplication() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [0x18];
            //TODO(>timeout management): decide what to do if we timeout
            // because it could mean that bootloader didn't receive,
            // startApplication packet.
            yield this.send(data);
        });
    }
    getBoardInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [0x11];
            const response = yield this.send(data);
            // We are not interested in checking command byte because it cannot fail
            const [minor, major, modelNumber] = response.slice(1);
            const model = `HP${modelNumber}00`;
            const version = `${major}.${minor}`;
            return { model, version };
        });
    }
    getFirmwareVersion() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [0x12];
            const response = yield this.send(data);
            const [minor, major] = response.slice(1);
            return `${major}.${minor}`;
        });
    }
    setFlashAddress(address) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [0x21];
            data.push((address >> 24) & 0xff);
            data.push((address >> 16) & 0xff);
            data.push((address >> 8) & 0xff);
            data.push(address & 0xff);
            const response = yield this.send(data);
            if (response && response[0] === 0x21) {
                throw new Error(`Address ${address} is out of valid range.`);
            }
        });
    }
    writeFlash(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const payload = [0x22, ...data];
            const response = yield this.send(payload);
            if (response && response[0] === 0x22) {
                throw new Error('Unable to flash data');
            }
        });
    }
    //NOTE: send does not check if command is failed, instead it leaves
    // this task to the caller. That's why it should provide some
    // way to discern timeout errors from packet errors I think.
    // However, until we manage timeout, we will keep this implementation
    send(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const packet = this.appendCrc(data);
            let attempts = 1;
            let response = null;
            // If response is invalid (i.e. its CRC is invalid) or contains a CRC error
            // (i.e. packet sent had wrong CRC) we immediately resend same packet.
            // Number of attempts should be configurable I think...
            while (!response && attempts <= 10) {
                response = yield this.connection.send(packet);
                debug(`Received response: ${response}`);
                if (!this.isValid(response) || this.crcError(response)) {
                    debug(`Invalid response #${attempts}: ${util_1.default.inspect(response)}`);
                    response = null;
                    attempts++;
                }
            }
            if (!response) {
                throw new Error('Timeout occured before receiving valid response.');
            }
            return response;
        });
    }
    *split(data) {
        while (data.length > 0) {
            let chunk = data.splice(0, constants_1.MAX_DATA_LENGTH);
            yield Buffer.from(chunk);
        }
    }
    appendCrc(data) {
        const crcBytes = crc.computeAsBytes(data);
        return Buffer.from(data.concat(crcBytes));
    }
    isValid(response) {
        // Last two bytes contain CRC value (res[-2] -> msb, res[-1] -> lsb)
        const expected = response.pop() + (response.pop() << 8);
        const actual = crc.computeAsValue(response);
        return expected === actual;
    }
    crcError(response) {
        return response[0] === constants_1.CRC_ERROR;
    }
}
exports.default = Programmer;
